use bevy::prelude::*;

#[cfg(target_family = "wasm")]
mod fullviewport;
#[cfg(target_family = "wasm")]
use fullviewport::FullViewportPlugin;

fn main() {
    #[cfg(all(target_family = "wasm", debug_assertions))]
    console_error_panic_hook::set_once();

    let mut app = App::new();

    #[cfg(target_family = "wasm")]
    app.add_plugin(FullViewportPlugin);

    app.add_plugins(DefaultPlugins)
        .add_startup_system(setup)
        .add_startup_system(start_background_audio)
        .add_system(rotate)
        .run();
}

#[derive(Component)]
struct Rotatable {
    speed: f32,
}

fn setup(
    mut commands: Commands,
    mut meshes: ResMut<Assets<Mesh>>,
    mut materials: ResMut<Assets<StandardMaterial>>,
) {
    commands.insert_resource(ClearColor(Color::BLACK));
    commands.insert_resource(AmbientLight {
        color: Color::VIOLET,
        brightness: 0.05,
    });

    commands.spawn_bundle(Camera3dBundle {
        transform: Transform::from_xyz(0.0, 25.0, 50.0).looking_at(Vec3::ZERO, Vec3::Y),
        ..Default::default()
    });

    commands.spawn_bundle(PbrBundle {
        mesh: meshes.add(Mesh::from(shape::Plane { size: 1000.0 })),
        material: materials.add(StandardMaterial {
            base_color: Color::BLACK,
            //metallic: 1.0,
            reflectance: 1.0,
            //perceptual_roughness: 0.0,
            ..Default::default()
        }),
        ..Default::default()
    });

    commands
        .spawn_bundle(PbrBundle {
            mesh: meshes.add(Mesh::from(shape::Cube { size: 4.0 })),
            material: materials.add(StandardMaterial {
                base_color: Color::BLACK,
                metallic: 1.0,
                //reflectance: 1.0,
                //perceptual_roughness: 0.0,
                double_sided: true,
                ..Default::default()
            }),
            transform: Transform::from_translation(Vec3::new(0.0, 8.0, 0.0)),
            ..Default::default()
        })
        .insert(Rotatable { speed: 0.1 });


    commands.spawn_bundle(DirectionalLightBundle {
        directional_light: DirectionalLight {
            color: Color::PINK,
            illuminance: 100000.0,
            shadows_enabled: true,
            ..Default::default()
        },
        transform: Transform {
            translation: Vec3::new(0.0, 16.0, 0.0),
            rotation: Quat::from_rotation_x(-std::f32::consts::PI),
            ..Default::default()
        },
        ..Default::default()
    });
}

fn start_background_audio(asset_server: Res<AssetServer>, audio: Res<Audio>) {
    audio.play_with_settings(asset_server.load("sounds/bg.ogg"), PlaybackSettings::LOOP);
}

fn rotate(mut rotatables: Query<(&mut Transform, &Rotatable)>, timer: Res<Time>) {
    for (mut transform, rotatable) in rotatables.iter_mut() {
        let rotation = 2.0 * std::f32::consts::PI * timer.delta_seconds();
        let rotation_change = Quat::from_scaled_axis(Vec3::new(
            rotation * rotatable.speed,
            rotation * rotatable.speed * -2.,
            rotation * rotatable.speed,
        ));
        transform.rotate(rotation_change);
    }
}
